import argparse

from kpc_nifi_utils.common.singleton import singleton


class Args:
    def __init__(self, *args):
        self._wrapper = ArgumentParserWrapper()
        for arg in args:
            if isinstance(arg, str):
                self._wrapper.add('--{}'.format(arg), type=str)

    def add(self, *args, **kwargs):
        if self._wrapper.is_parse():
            raise ValueError("Can not add args config when parser is already initiate")

        self._wrapper.add(*args, **kwargs)
        return self

    def get(self, key):
        if not self._wrapper.is_parse():
            raise ValueError(
                "Can not get args value because parser not initiate, calling initiate() and then try again")

        try:
            return vars(self._wrapper.get_args()).get(key, None)
        except Exception as e:
            print(e)

    def initiate(self):
        self._wrapper.parse()
        return self

    def print_help(self):
        print(self._wrapper.print_help())


@singleton
class ArgumentParserWrapper:
    def __init__(self):
        self._args = None
        self._parser = argparse.ArgumentParser()
        self._is_parse = False

    def add(self, *args, **kwargs):
        self._parser.add_argument(*args, **kwargs)
        return self

    def get_args(self):
        return self._args

    def parse(self):
        self._args = self._parser.parse_args()
        self._is_parse = True
        return self

    def is_parse(self):
        return self._is_parse

    def print_help(self):
        print(self._parser.format_help())



"""MongoDB Parameter Constants (MongoArgs)"""
MONGO_USERNAME = 'MONGO_USERNAME'
MONGO_PASSWORD = 'MONGO_PASSWORD'
MONGO_HOST = 'MONGO_HOST'
MONGO_PORT = 'MONGO_PORT'
MONGO_DBNAME = 'MONGO_DBNAME'
MONGO_REPLICA_SET = 'MONGO_REPLICA_SET'

"""Microsoft SQL Server Parameter Constants (MSSqlArgs)"""
MSSQL_USERNAME = 'MSSQL_USERNAME'
MSSQL_PASSWORD = 'MSSQL_PASSWORD'
MSSQL_HOST = 'MSSQL_HOST'
MSSQL_PORT = 'MSSQL_PORT'
MSSQL_DBNAME = 'MSSQL_DBNAME'

"""PostgreSQL Parameter Constants (PostgreSQLArgs)"""
POSTGRESQL_USERNAME = 'POSTGRESQL_USERNAME'
POSTGRESQL_PASSWORD = 'POSTGRESQL_PASSWORD'
POSTGRESQL_HOST = 'POSTGRESQL_HOST'
POSTGRESQL_PORT = 'POSTGRESQL_PORT'
POSTGRESQL_DBNAME = 'POSTGRESQL_DBNAME'

"""Snowflake Parameter Constants (SnowflakeArgs)"""
SNOWFLAKE_ENV = 'SNOWFLAKE_ENV'
SNOWFLAKE_USERNAME = 'SNOWFLAKE_USERNAME'
SNOWFLAKE_PASSWORD = 'SNOWFLAKE_PASSWORD'
SNOWFLAKE_ACCOUNT = 'SNOWFLAKE_ACCOUNT'
SNOWFLAKE_WAREHOUSE = 'SNOWFLAKE_WAREHOUSE'
SNOWFLAKE_ROLE = 'SNOWFLAKE_ROLE'

"""AWS Parameter Constants (AwsArgs, S3Args)"""
AWS_KEY = 'AWS_KEY'
AWS_SECRET = 'AWS_SECRET'

"""S3 Parameter Constants (S3Args)"""
AWS_S3_BUCKET = 'AWS_S3_BUCKET'



import json
import sys

from kpc_nifi_utils.common.singleton import singleton


@singleton
class Command:
    def send(self, *args, **kwargs):
        print(*args, file=sys.stdout, **kwargs)

    def receive(self):
        try:
            return json.load(sys.stdin)
        except Exception as e:
            print(e)
            return None


class Sender:
    def __init__(self):
        self._command = Command()
        self._config = {}
        self._data = {}

    def config_set(self, key, value):
        self._config[key] = value
        return self

    def data_set(self, key, value):
        self._data[key] = value
        return self

    def configs_set(self, config: dict):
        self._config.update(config)
        return self

    def datum_set(self, datum: dict):
        self._data.update(datum)
        return self

    def send(self):
        self._command.send(self.__str__())

    def __str__(self):
        return json.dumps({'config': self._config, 'data': self._data})


class Receiver:
    def __init__(self):
        self._command = Command()
        self._config = None
        self._data = None
        self._receiver = None

    def receive(self):
        self._receiver = Command().receive()

        try:
            self._config = self._receiver['config']
        except Exception as e:
            print(e)

        try:
            self._data = self._receiver['data']
        except Exception as e:
            print(e)

    def config_get(self):
        if self._config is not None:
            return self._config

        return None

    def config_get_by_key(self, key):
        if self._config is not None:
            try:
                return self._config[key]
            except Exception as e:
                print(e)
                return None

    def data_get(self):
        if self._data is not None:
            return self._data

        return None

    def data_get_by_key(self, key):
        if self._data is not None:
            try:
                return self._data[key]
            except Exception as e:
                print(e)
                return None

    def receiver_get(self):
        if self._receiver is not None:
            return self._receiver

        return None

    def receiver_get_by_key(self, key):
        if self._receiver is not None:
            try:
                return self._receiver[key]
            except Exception as e:
                print(e)
                return None



import json
import urllib.parse as urlparse

from pymongo import MongoClient
from pymongo.cursor import Cursor

import kpc_nifi_utils.shared.constants as constants
from kpc_nifi_utils.args.mongodb import MongoArgs
from kpc_nifi_utils.common.json_encoder import EnhancedJSONEncoder


class MongoConnector:
    def __init__(self, connector_args: MongoArgs = None, map_setting: dict = None):
        self._args = connector_args
        self._map_setting = map_setting

        self._username = None
        self._password = None
        self._host = None
        self._port = None
        self._db_name = None
        self._replica_set = None

        if self._args is not None:
            self._username = self._args.get_username()
            self._password = self._args.get_password()
            self._host = self._args.get_host()
            self._port = self._args.get_port()
            self._db_name = self._args.get_dbname()
            self._replica_set = self._args.get_replicaset()

        if self._map_setting is not None:
            self._username = self._username if self._username else self._map_setting.get(constants.MONGO_USERNAME)
            self._password = self._password if self._password else self._map_setting.get(constants.MONGO_PASSWORD)
            self._host = self._host if self._host else self._map_setting.get(constants.MONGO_HOST)
            self._port = self._port if self._port else self._map_setting.get(constants.MONGO_PORT)
            self._db_name = self._db_name if self._db_name else self._map_setting.get(constants.MONGO_DBNAME)
            self._replica_set = self._replica_set if self._replica_set else self._map_setting.get(
                constants.MONGO_REPLICA_SET)

    def get_db(self,
               host=None,
               port=None,
               document_class=dict,
               tz_aware=None,
               connect=None,
               dbname=None,
               **kwargs):

        if host is None and self._args is None and self._map_setting is None:
            raise ValueError("Can not get db because there is no setting initialize or pass")

        dbname = dbname if dbname else self._db_name

        if host is None:
            connection_string = 'mongodb://'
            if self._username and self._password:
                connection_string += '{}:{}@'.format(urlparse.quote_plus(self._username),
                                                     urlparse.quote_plus(self._password))

            connection_string += '{}'.format(self._host)

            if self._port:
                connection_string += ':{}'.format(self._port)

            host = '{}'.format(connection_string)

            if self._replica_set:
                host += '/?replicaSet={}'.format(self._replica_set)

        client = MongoClient(host,
                             port,
                             document_class,
                             tz_aware,
                             connect,
                             **kwargs)

        return client.get_database(dbname)

    @staticmethod
    def parse_as_json(data):
        data_parse = data
        if isinstance(data, dict):
            data_parse = [data]
        elif isinstance(data, Cursor):
            data_parse = list(data)

        return json.dumps(data_parse, cls=EnhancedJSONEncoder)



import kpc_nifi_utils.shared.constants as constants
from kpc_nifi_utils.common.args_parser import Args


class AwsArgs(Args):
    def __init__(self, *args):
        super().__init__(*args)

    def get_key(self):
        return self.get(constants.AWS_KEY)

    def get_secret(self):
        return self.get(constants.AWS_SECRET)



import json

import pytds

import kpc_nifi_utils.shared.constants as constants
from kpc_nifi_utils.args.mssql import MSSqlArgs
from kpc_nifi_utils.common.json_encoder import EnhancedJSONEncoder


class MSSqlConnector:
    def __init__(self, connector_args: MSSqlArgs = None, map_setting: dict = None):
        self._args = connector_args
        self._map_setting = map_setting

        self._username = None
        self._password = None
        self._host = None
        self._port = None
        self._db_name = None

        if self._args is not None:
            self._username = self._args.get_username()
            self._password = self._args.get_password()
            self._host = self._args.get_dsn()
            self._port = self._args.get_port()
            self._db_name = self._args.get_dbname()

        if self._map_setting is not None:
            self._username = self._username if self._username else self._map_setting.get(constants.MSSQL_USERNAME)
            self._password = self._password if self._password else self._map_setting.get(constants.MSSQL_PASSWORD)
            self._host = self._host if self._host else self._map_setting.get(constants.MSSQL_HOST)
            self._port = self._port if self._port else self._map_setting.get(constants.MSSQL_PORT)
            self._db_name = self._db_name if self._db_name else self._map_setting.get(constants.MSSQL_DBNAME)

    def connect(self,
                dsn=None, database=None, user=None, password=None, timeout=None, login_timeout=15, as_dict=None,
                appname=None, port=None, tds_version=1946157060, autocommit=False, blocksize=4096, use_mars=False,
                auth=None, readonly=False, load_balancer=None, use_tz=None, bytes_to_unicode=True, row_strategy=None,
                failover_partner=None, server=None, cafile=None, validate_host=True, enc_login_only=False):

        if self._args is None \
                and self._map_setting is None \
                and dsn is None \
                and database is None \
                and user is None \
                and password is None \
                and port is None:
            raise ValueError("Can not get db because there is no setting initialize or pass")

        dsn = dsn if dsn else self._host
        user = user if user else self._username
        password = password if password else self._password
        port = port if port else self._port
        database = database if database else self._db_name

        if as_dict is None:
            as_dict = True

        return pytds.connect(dsn=dsn, database=database, user=user, password=password, timeout=timeout,
                             login_timeout=login_timeout, as_dict=as_dict, appname=appname, port=port,
                             tds_version=tds_version, autocommit=autocommit, blocksize=blocksize, use_mars=use_mars,
                             auth=auth, readonly=readonly, load_balancer=load_balancer, use_tz=use_tz,
                             bytes_to_unicode=bytes_to_unicode, row_strategy=row_strategy,
                             failover_partner=failover_partner, server=server, cafile=cafile,
                             validate_host=validate_host, enc_login_only=enc_login_only)

    @staticmethod
    def parse_as_json(data):
        data_parse = data
        if isinstance(data, dict):
            data_parse = [data]

        return json.dumps(data_parse, cls=EnhancedJSONEncoder)



from kpc_nifi_utils.common.args_parser import Args
from kpc_nifi_utils.shared import constants


class SnowflakeArgs(Args):
    def __init__(self, *args):
        super().__init__(*args)

    def get_env(self):
        return self.get(constants.SNOWFLAKE_ENV)

    def get_username(self):
        return self.get(constants.SNOWFLAKE_USERNAME)

    def get_password(self):
        return self.get(constants.SNOWFLAKE_PASSWORD)

    def get_account(self):
        return self.get(constants.SNOWFLAKE_ACCOUNT)

    def get_warehouse(self):
        return self.get(constants.SNOWFLAKE_WAREHOUSE)

    def get_role(self):
        return self.get(constants.SNOWFLAKE_ROLE)
